use gtk::prelude::*;

use relm4::component::{AsyncComponent, AsyncComponentParts};
use relm4::factory::FactoryVecDeque;
use relm4::gtk;
use relm4::{AsyncComponentSender, RelmListBoxExt};

use super::autosuggestion_element::AutosuggestionElementModel;

pub struct AutosuggestionPopoverModel {
    hidden: bool,
    suggestions: Vec<String>,
    suggestion_rows: FactoryVecDeque<AutosuggestionElementModel>,
}

#[derive(Debug)]
pub enum AutosuggestionPopoverInput {
    Show,
    Hide,
    SetSuggestions(Vec<String>),
    SuggestionIndexSelected(i32),
    None,
}

#[derive(Debug)]
pub enum AutosuggestionPopoverOutput {
    SuggestionSelected(String),
}

#[relm4::component(pub, async)]
impl AsyncComponent for AutosuggestionPopoverModel {
    type Init = ();
    type Input = AutosuggestionPopoverInput;
    type Output = AutosuggestionPopoverOutput;
    type CommandOutput = ();

    view! {
        gtk::Popover {
            #[watch]
            set_visible: !model.hidden,
            set_has_arrow: false,
            model.suggestion_rows.widget() -> &relm4::gtk::ListBox {
                set_hexpand: true,
                // set_orientation: gtk::Orientation::Vertical,
                // set_spacing: 5,
                connect_row_activated[sender] => move |list_box, row| {
                    let index = list_box.index_of_child(row).unwrap();
                    sender.input(AutosuggestionPopoverInput::SuggestionIndexSelected(index));
                },
            },
        }
    }

    async fn init(
        _params: Self::Init,
        root: Self::Root,
        sender: AsyncComponentSender<Self>,
    ) -> AsyncComponentParts<Self> {
        let suggestion_rows = FactoryVecDeque::builder()
            .launch(gtk::ListBox::default())
            .forward(sender.input_sender(), |_| AutosuggestionPopoverInput::None);

        let model = AutosuggestionPopoverModel {
            hidden: true,
            suggestions: Vec::new(),
            suggestion_rows,
        };
        let widgets = view_output!();

        AsyncComponentParts { model, widgets }
    }

    // TODO: init_loading_widgets

    async fn update(
        &mut self,
        msg: Self::Input,
        _sender: AsyncComponentSender<Self>,
        _root: &Self::Root,
    ) {
        match msg {
            AutosuggestionPopoverInput::Show => self.hidden = false,
            AutosuggestionPopoverInput::Hide => self.hidden = true,
            AutosuggestionPopoverInput::SetSuggestions(suggestions) => {
                self.suggestion_rows.guard().clear();
                for suggestion in suggestions.iter() {
                    self.suggestion_rows
                        .guard()
                        .push_back(suggestion.to_string());
                }
                self.suggestions = suggestions;
            }
            AutosuggestionPopoverInput::None => {}
            AutosuggestionPopoverInput::SuggestionIndexSelected(index) => {
                let suggestion = self.suggestions.get(index as usize).unwrap();
                _sender
                    .output(AutosuggestionPopoverOutput::SuggestionSelected(
                        suggestion.to_string(),
                    ))
                    .unwrap();
            }
        }
    }
}
